==> rtlinux-2.1.tar.gz: rtlinux-2.1/include/rtl_mutex.h <==
/*
 * RT-Linux mutex implementation
 *
 * Written by Michael Barabanov
 * Copyright (C) VJY Associates LLC, 1999
 * Released under the terms of the GPL
 *
 */

#ifndef __RTL_MUTEX__
#define __RTL_MUTEX__

#ifdef __KERNEL__

#include <rtl_conf.h>
#ifdef CONFIG_RTL_MUTEX_SUPPORT

#include <rtl_sched.h>
#include <rtl_waitqueue.h>

enum { PTHREAD_MUTEX_NORMAL, PTHREAD_MUTEX_RECURSIVE, PTHREAD_MUTEX_ERRORCHECK,
	      PTHREAD_MUTEX_DEFAULT = PTHREAD_MUTEX_NORMAL };

typedef struct {
	int type;
	int pshared;
} pthread_mutexattr_t;

enum { PTHREAD_PROCESS_SHARED, PTHREAD_PROCESS_PRIVATE };

extern inline int pthread_mutexattr_getpshared(const pthread_mutexattr_t *attr,
		    int *pshared)
{
	*pshared = attr->pshared;
	return 0;
}

extern inline int pthread_mutexattr_setpshared(pthread_mutexattr_t *attr,
		    int pshared)
{
	if (pshared != PTHREAD_PROCESS_SHARED
			&& pshared != PTHREAD_PROCESS_PRIVATE) {
		return EINVAL;
	}
	attr->pshared = pshared;
	return 0;
}


extern inline int pthread_mutexattr_init(pthread_mutexattr_t *attr)
{
	attr->type = PTHREAD_MUTEX_DEFAULT;
	attr->pshared = PTHREAD_PROCESS_PRIVATE;
	return 0;
}

extern inline int pthread_mutexattr_destroy(pthread_mutexattr_t *attr)
{
	return 0;
}

extern int pthread_mutexattr_settype(pthread_mutexattr_t *attr, int type);

extern inline int pthread_mutexattr_gettype(const pthread_mutexattr_t *attr, int *type)
{
	*type = attr->type;
	return 0;
}


typedef struct {
	int count;
	spinlock_t lock;
	int type;
	RTL_WQ wq;
} pthread_mutex_t;



#define PTHREAD_MUTEX_INITIALIZER { 0, SPIN_LOCK_UNLOCKED, PTHREAD_MUTEX_DEFAULT, RTL_WQ_INITIALIZER }

extern int pthread_mutex_init(pthread_mutex_t *mutex,
		    const pthread_mutexattr_t *attr);
extern int pthread_mutex_destroy(pthread_mutex_t *mutex);

extern int pthread_mutex_lock(pthread_mutex_t *mutex);
extern int pthread_mutex_trylock(pthread_mutex_t *mutex);
extern int pthread_mutex_unlock(pthread_mutex_t *mutex);

#endif
#endif


/* Common routines for implementing IPC */

extern inline void rtl_thread_mark_self_suspended(void)
{
	pthread_self()->state = RTL_THREAD_SUSPENDED;
}

extern inline void rtl_thread_pend_wakeup(pthread_t thread)
{
	set_bit (RTL_SIGNAL_WAKEUP, &thread->pending_signals);
}

extern inline void rtl_reschedule2(int cpu)
{
#ifdef __SMP__
	if (cpu != rtl_getcpuid())
		rtl_reschedule(cpu);
	else
#endif
		rtl_schedule();
}

#endif

==> rtlinux-2.3-pre1.tar.gz: rtlinux-2.3/include/rtl_mutex.h <==
/*
 * RT-Linux mutex implementation
 *
 * Written by Michael Barabanov
 * Copyright (C) VJY Associates LLC, 1999
 * Released under the terms of the GPL
 *
 */

#ifndef __RTL_MUTEX__
#define __RTL_MUTEX__

#ifdef __KERNEL__

#include <rtl_conf.h>
#ifdef CONFIG_RTL_MUTEX_SUPPORT

#include <rtl_sched.h>

enum { PTHREAD_MUTEX_NORMAL, PTHREAD_MUTEX_RECURSIVE, PTHREAD_MUTEX_ERRORCHECK,
	      PTHREAD_MUTEX_DEFAULT = PTHREAD_MUTEX_NORMAL };

typedef struct {
	int type;
	int pshared;
} pthread_mutexattr_t;

enum { PTHREAD_PROCESS_SHARED, PTHREAD_PROCESS_PRIVATE };

/* TODO make this simpler */
typedef struct {
	pthread_t owner;
	int busy;
	int count;
	int valid;
	spinlock_t lock;
	int type;
	pthread_t waiters[CONFIG_RTL_MUTEX_MAX_WAITERS];
} pthread_mutex_t;

#define PTHREAD_MUTEX_INITIALIZER { NULL,0,0,1, SPIN_LOCK_UNLOCKED, PTHREAD_MUTEX_DEFAULT,{ 0 } }


/* Mutex types are not good things.  */

static inline int pthread_mutexattr_settype(pthread_mutexattr_t *attr, int type)
{
	return ((attr->type == PTHREAD_MUTEX_DEFAULT) ? 0: -EINVAL);
}



static inline int pthread_mutexattr_getpshared(const pthread_mutexattr_t *attr,
		    int *pshared)
{
	*pshared = attr->pshared;
	return 0;
}

/*TODO something sensible with this. We don't worry about
  shared and not shared yet.
  */
extern inline int pthread_mutexattr_setpshared(pthread_mutexattr_t *attr,
		    int pshared)
{
	if (pshared != PTHREAD_PROCESS_SHARED
			&& pshared != PTHREAD_PROCESS_PRIVATE) {
		return EINVAL;
	}
	attr->pshared = pshared;
	return 0;
}


extern inline int pthread_mutexattr_init(pthread_mutexattr_t *attr)
{
	attr->type = PTHREAD_MUTEX_DEFAULT;
	attr->pshared = PTHREAD_PROCESS_PRIVATE;
	return 0;
}

extern inline int pthread_mutexattr_destroy(pthread_mutexattr_t *attr)
{
	return 0;
}

extern int pthread_mutexattr_settype(pthread_mutexattr_t *attr, int type);

extern inline int pthread_mutexattr_gettype(const pthread_mutexattr_t *attr, int *type)
{
	*type = attr->type;
	return 0;
}


extern int pthread_mutex_init(pthread_mutex_t *mutex,
		    const pthread_mutexattr_t *attr);
extern inline int pthread_mutex_destroy(pthread_mutex_t *mutex);

extern int pthread_mutex_lock(pthread_mutex_t *mutex);
extern int pthread_mutex_trylock(pthread_mutex_t *mutex);
extern int pthread_mutex_unlock(pthread_mutex_t *mutex);

#endif
#endif

#endif
